@@ -374,6 +374,7 @@ main(int argc, char* argv[])
 	(void)unsetenv("NOTIFY_SOCKET");
 #endif /* HAVE_SYSTEMD */
 
+	checklock_start();
 	log_init(NULL, 0, NULL);
 	/* determine commandline options for the daemon */
 	pass_argc = 1;
@@ -604,3 +605,13 @@ int squelch_err_ssl_handshake(unsigned long ATTR_UNUSED(err))
 {
 	return 0;
 }
+
+void listen_setup_locks(void)
+{
+	/* nothing */
+}
+
+void listen_desetup_locks(void)
+{
+	/* nothing */
+}
